@@ -1148,6 +1148,30 @@ wd_set_node_mask (WD_PACKET_NO packet_no, int *node_id_set, int count)
 	return rtn;
 }
 
+#ifdef USE_SSL
+/* HMAC SHA-256*/
+static void calculate_hmac_sha256(const char *data, int len, char *buf)
+{
+	char* key = pool_config->wd_authkey;
+	char str[WD_AUTH_HASH_LEN/2];
+	
+	unsigned int res_len = WD_AUTH_HASH_LEN;
+	
+	HMAC_CTX ctx;
+	HMAC_CTX_init(&ctx);
+	HMAC_Init_ex(&ctx, key, strlen(key), EVP_sha256(), NULL);
+	HMAC_Update(&ctx, (unsigned char*)data, len);
+	HMAC_Final(&ctx, (unsigned char*)str, &res_len);
+	HMAC_CTX_cleanup(&ctx);
+	bytesToHex(str,32,buf);
+	buf[WD_AUTH_HASH_LEN] = '\0';
+}
+void
+wd_calc_hash(const char *str, int len, char *buf)
+{
+	calculate_hmac_sha256(str, len, buf);
+}
+#else
 /* calculate hash for authentication using packet contents */
 void
 wd_calc_hash(const char *str, int len, char *buf)
@@ -1157,34 +1181,38 @@ wd_calc_hash(const char *str, int len, char *buf)
 	size_t pass_len;
 	size_t username_len;
 	size_t authkey_len;
-
+	char tmp_buf[(MD5_PASSWD_LEN+1)*2];
 	/* use first half of authkey as username, last half as password */
 	authkey_len = strlen(pool_config->wd_authkey);
-
+	
 	if (len <= 0 || authkey_len <= 0)
 		goto wd_calc_hash_error;
-
+	
 	username_len = authkey_len / 2;
 	pass_len = authkey_len - username_len;
 	if ( snprintf(username, username_len + 1, "%s", pool_config->wd_authkey) < 0
-	  || snprintf(pass, pass_len + 1, "%s", pool_config->wd_authkey + username_len) < 0)
+		|| snprintf(pass, pass_len + 1, "%s", pool_config->wd_authkey + username_len) < 0)
 		goto wd_calc_hash_error;
-
+	
 	/* calculate hash using md5 encrypt */
-	if (! pool_md5_encrypt(pass, username, strlen(username), buf + MD5_PASSWD_LEN + 1))
+	if (! pool_md5_encrypt(pass, username, strlen(username), tmp_buf + MD5_PASSWD_LEN + 1))
 		goto wd_calc_hash_error;
 	buf[(MD5_PASSWD_LEN+1)*2-1] = '\0';
-
-	if (! pool_md5_encrypt(buf+MD5_PASSWD_LEN+1, str, len, buf))
+	
+	if (! pool_md5_encrypt(tmp_buf+MD5_PASSWD_LEN+1, str, len, tmp_buf))
 		goto wd_calc_hash_error;
+	
+	memcpy(buf, tmp_buf, MD5_PASSWD_LEN);
 	buf[MD5_PASSWD_LEN] = '\0';
-
+	
 	return;
-
+	
 wd_calc_hash_error:
 	buf[0] = '\0';
 	return;
 }
+#endif
+
 
 int
 wd_packet_to_string(WdPacket *pkt, char *str, int maxlen)
